package org.processmining.plugins.log.ui.logdialog;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;

import javax.swing.AbstractListModel;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContainer;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeList;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;

public class AttributesListModel extends AbstractListModel<String> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -719743180764113639L;
	protected XAttributeMap attributes;
	protected ArrayList<String> keys;

	protected AttributesListModel(XAttributeMap attributes) {
		if (attributes != null) {
			this.attributes = attributes;
			ArrayList<String> boolAttrs = new ArrayList<String>();
			ArrayList<String> discAttrs = new ArrayList<String>();
			ArrayList<String> contAttrs = new ArrayList<String>();
			ArrayList<String> listAttrs = new ArrayList<String>();
			ArrayList<String> containerAttrs = new ArrayList<String>();
			ArrayList<String> literAttrs = new ArrayList<String>();
			ArrayList<String> timeAttrs = new ArrayList<String>();
			ArrayList<String> otherAttrs = new ArrayList<String>();

			for (XAttribute attribute : attributes.values()) {
				if (attribute instanceof XAttributeBoolean)
					boolAttrs.add(attribute.getKey());
				else if (attribute instanceof XAttributeDiscrete)
					discAttrs.add(attribute.getKey());
				else if (attribute instanceof XAttributeContinuous)
					contAttrs.add(attribute.getKey());
				// XAttributeList and XAttributeContainer extend XAttributeLiteral, so their checks have to be performed first.
				else if (attribute instanceof XAttributeList)
					listAttrs.add(attribute.getKey());
				else if (attribute instanceof XAttributeContainer)
					containerAttrs.add(attribute.getKey());
				else if (attribute instanceof XAttributeLiteral)
					literAttrs.add(attribute.getKey());
				else if (attribute instanceof XAttributeTimestamp)
					timeAttrs.add(attribute.getKey());
				else
					otherAttrs.add(attribute.getKey());

			}
			Collections.sort(boolAttrs);
			Collections.sort(contAttrs);
			Collections.sort(discAttrs);
			Collections.sort(listAttrs);
			Collections.sort(containerAttrs);
			Collections.sort(literAttrs);
			Collections.sort(timeAttrs);
			Collections.sort(otherAttrs);

			keys = new ArrayList<String>();
			if (boolAttrs.size() > 0) {
				keys.add("BOOLEAN TYPED");
				keys.addAll(boolAttrs);
			}
			if (contAttrs.size() > 0) {
				keys.add("CONTINOUS TYPED");
				keys.addAll(contAttrs);
			}
			if (discAttrs.size() > 0) {
				keys.add("DISCRETE TYPED");
				keys.addAll(discAttrs);
			}
			if (listAttrs.size() > 0) {
				keys.add("LIST TYPED");
				keys.addAll(listAttrs);
			}
			if (containerAttrs.size() > 0) {
				keys.add("CONTAINER TYPED");
				keys.addAll(containerAttrs);
			}
			if (literAttrs.size() > 0) {
				keys.add("LITERAL TYPED");
				keys.addAll(literAttrs);
			}
			if (timeAttrs.size() > 0) {
				keys.add("TIMESTAMP TYPED");
				keys.addAll(timeAttrs);
			}
			if (otherAttrs.size() > 0) {
				keys.add("OTHERS");
				keys.addAll(otherAttrs);
			}

		} else {
			this.attributes = null;
			keys = null;
		}
	}

	public AttributesListModel(XEvent ate) {
		this(ate == null ? null : ate.getAttributes());
	}

	public AttributesListModel(XTrace instance) {
		this(instance == null ? null : instance.getAttributes());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public String getElementAt(int index) {
		XAttribute attr = attributes.get(keys.get(index));
		if (attr != null)
			return "<html>" + printAttribute(attr, 1, false) + "</html>";
		else
			return "<html><b><FONT COLOR=\"FFFF00\"><c>" + keys.get(index) + "</c></font></b></html>\"";
	}

	private String printAttribute(XAttribute attr, int indent, boolean meta) {
		String val = printIndent(indent);
		if (!meta)
			val += "<b><font color=\"FFFFFF\">" + attr.getKey() + "</font></b>:  ";
		else
			val += "<i><b><font color=\"FFFFCC\">" + attr.getKey() + "</font></b></i>:  ";

		if (attr instanceof XAttributeList) {
			XAttributeList attrList = (XAttributeList) attr;
			if (attrList.getCollection().size() > 0) {
				val += "<br/>";
				Iterator<XAttribute> childIt = attrList.getCollection().iterator();
				while (childIt.hasNext()) {
					XAttribute childAttr = childIt.next();
					val += printAttribute(childAttr, indent + 3, false);
					if (childIt.hasNext())
						val += "<br/>";
				}
			}
		} else if (attr instanceof XAttributeTimestamp) {
			Date timestamp = ((XAttributeTimestamp) attr).getValue();
			val += new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss").format(timestamp);
		} else {
			val += attr.toString();
		}

		if (attr.hasAttributes() && !(attr instanceof XAttributeList)) {
			val += "<br/>";
			Iterator<String> metaIt = attr.getAttributes().keySet().iterator();
			while (metaIt.hasNext()) {
				String metaAttrKey = metaIt.next();
				XAttribute metaAttr = attr.getAttributes().get(metaAttrKey);
				val += printAttribute(metaAttr, indent + 3, true);
				if (metaIt.hasNext())
					val += "<br/>";
			}
		}

		return val;
	}

	private String printIndent(int indent) {
		String val = "";
		for (int i = 0; i < indent; i++) {
			val += "&nbsp;";
		}
		return val;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		if (keys != null) {
			return keys.size();
		} else {
			return 0;
		}
	}

}