package org.processmining.plugins.log.ui.logdialog;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

import org.deckfour.xes.extension.std.XExtendedEvent;
import org.deckfour.xes.model.XEvent;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractivityContext;

import com.fluxicon.slickerbox.components.GradientPanel;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class EventCellRenderer extends GradientPanel implements ListCellRenderer<Pair<EID, XEvent>> {
	private static final long serialVersionUID = 1L;
	
	public static final Color COLOR_TAGGED_TOP = new Color(100, 100, 0);
	public static final Color COLOR_TAGGED_BOTTOM = new Color(60, 60, 0);
	public static final Color COLOR_TAGGED_SELECTED = new Color(200, 0, 0);
	
	protected int height = 60;
	protected DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss.SSS");

	protected JLabel nameLabel;
	protected JLabel numberLabel;
	protected JLabel originatorLabel;
	protected JLabel typeLabel;
	protected JLabel timestampLabel;

	private InteractivityContext interactivityContext;
	
	/**
	 * @param colorTop
	 * @param colorBottom
	 */
	public EventCellRenderer(InteractivityContext ic) {
		super(LogPreviewUI.COLOR_LIST_BG, LogPreviewUI.COLOR_LIST_BG_LOWER);
		
		this.interactivityContext = ic;
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setMinimumSize(new Dimension(180, height));
		setMaximumSize(new Dimension(500, height));
		setPreferredSize(new Dimension(250, height));
		nameLabel = new JLabel("name");
		nameLabel.setOpaque(false);
		nameLabel.setForeground(LogPreviewUI.COLOR_LIST_FG);
		nameLabel.setFont(nameLabel.getFont().deriveFont(13f));
		nameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		nameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		nameLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		originatorLabel = new JLabel("originator");
		originatorLabel.setOpaque(false);
		originatorLabel.setForeground(LogPreviewUI.COLOR_LIST_FG);
		originatorLabel.setFont(originatorLabel.getFont().deriveFont(9f));
		numberLabel = new JLabel("#");
		numberLabel.setOpaque(false);
		numberLabel.setForeground(LogPreviewUI.COLOR_LIST_FG);
		numberLabel.setFont(numberLabel.getFont().deriveFont(9f));
		typeLabel = new JLabel("type");
		typeLabel.setOpaque(false);
		typeLabel.setForeground(LogPreviewUI.COLOR_LIST_FG);
		typeLabel.setFont(typeLabel.getFont().deriveFont(11f));
		timestampLabel = new JLabel("timestamp");
		timestampLabel.setOpaque(false);
		timestampLabel.setForeground(LogPreviewUI.COLOR_LIST_FG);
		timestampLabel.setFont(timestampLabel.getFont().deriveFont(12f));
		timestampLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		timestampLabel.setHorizontalAlignment(SwingConstants.CENTER);
		timestampLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.X_AXIS));
		middlePanel.setOpaque(false);
		middlePanel.setBorder(BorderFactory.createEmptyBorder());
		middlePanel.add(Box.createHorizontalGlue());
		middlePanel.add(numberLabel);
		middlePanel.add(Box.createHorizontalStrut(10));
		middlePanel.add(typeLabel);
		middlePanel.add(Box.createHorizontalStrut(10));
		middlePanel.add(originatorLabel);
		middlePanel.add(Box.createHorizontalGlue());
		this.add(Box.createVerticalGlue());
		this.add(nameLabel);
		this.add(Box.createVerticalGlue());
		this.add(middlePanel);
		this.add(Box.createVerticalGlue());
		this.add(timestampLabel);
		this.add(Box.createVerticalGlue());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing
	 * .JList, java.lang.Object, int, boolean, boolean)
	 */
	public Component getListCellRendererComponent(JList<? extends Pair<EID, XEvent>> list, Pair<EID, XEvent> value, int index, boolean isSelected,
			boolean cellHasFocus) {
		
		EID eventId = value.getFirst();
		XEvent event = value.getSecond();
		
		// configure yourself
		if (isSelected) {
			setColors(LogPreviewUI.COLOR_LIST_SELECTION_BG, LogPreviewUI.COLOR_LIST_SELECTION_BG_LOWER);
		} else {
			setColors(LogPreviewUI.COLOR_LIST_BG, LogPreviewUI.COLOR_LIST_BG_LOWER);
		}
		
		if(interactivityContext != null && interactivityContext.getManager() != null) {
			Set<EID> taggedEvents = interactivityContext.getManager().getAllTaggedEvents();
			
			if(taggedEvents.contains(eventId)) {
				
				if(isSelected) {
					setColors(COLOR_TAGGED_SELECTED, COLOR_TAGGED_BOTTOM);
				}
				else {
					setColors(COLOR_TAGGED_TOP, COLOR_TAGGED_BOTTOM);
				}
			}
		}
		
		XExtendedEvent ate = new XExtendedEvent(event);
		nameLabel.setText(ate.getName());
		typeLabel.setText(ate.getTransition());
		numberLabel.setText("#" + (index + 1));
		if (ate.getResource() != null) {
			originatorLabel.setText("@" + ate.getResource());
		} else {
			originatorLabel.setText("");
		}
		if (ate.getTimestamp() != null) {
			timestampLabel.setText(dateFormat.format(ate.getTimestamp()));
		} else {
			timestampLabel.setText("- no timestamp -");
		}
		return this;
	}
}