/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.log.ui.logdialog;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XLog;
import org.processmining.plugins.interactivevisualization.InteractivityContext;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class InspectorUI extends JPanel {

	private static final long serialVersionUID = -6450104278248630541L;
	protected SlickerOpenLogSettings parent;
	protected LogViewUI logViewUI;
	protected LogPreviewUI logPreviewUI;
	protected LogAttributesUI logAttributesUI;

	protected SlickerTabbedPane tabPane;

	/**
	 * @param summary
	 * @param title
	 * @param fgColor
	 * @param bgColor
	 * @param titleColor
	 */
	public InspectorUI(SlickerOpenLogSettings parent, InteractivityContext ic) {
		this.parent = parent;
		
		XLog log = parent.getLog();
		XLogInfo summary = parent.getLogInfo();
		
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
		tabPane = new SlickerTabbedPane("Log inspector", new Color(240, 240, 240, 230), new Color(0, 0, 0, 230),
				new Color(220, 220, 220, 150));
		logPreviewUI = new LogPreviewUI(log, summary, ic);
		tabPane.addTab("Browser", logPreviewUI);
		logViewUI = new LogViewUI(parent, log);
		tabPane.addTab("Explorer", logViewUI);
		logAttributesUI = new LogAttributesUI(log);
		tabPane.addTab("Log Attributes", logAttributesUI);

		this.add(tabPane);
	}

	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		
		logPreviewUI.updateVisualization(logChanged, tagsChanged);
		logViewUI.updateVisualization(logChanged, tagsChanged);
	}
}
