package org.processmining.plugins.log.ui.logdialog;

import java.awt.Color;
import java.awt.Component;
import java.util.SortedSet;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractivityContext;

public class InstanceCellRenderer extends JLabel implements ListCellRenderer<Pair<Integer, String>> {
	private static final long serialVersionUID = 1L;
	
	public static final Color COLOR_LIST_TAGGED_SELECTION_BG = Color.RED;
	public static final Color COLOR_LIST_TAGGED_BG = new Color(60, 60, 0);
	
	private InteractivityContext interactivityContext;
	
	public InstanceCellRenderer(InteractivityContext ic) {
		interactivityContext = ic;
		
		setForeground(LogPreviewUI.COLOR_LIST_FG);
		setOpaque(true);
		setBackground(LogPreviewUI.COLOR_LIST_BG);
	}
	
	public Component getListCellRendererComponent(JList<? extends Pair<Integer, String>> list, Pair<Integer, String> value,
			int index, boolean isSelected, boolean cellHasFocus) {
		int traceIdx = value.getFirst();
		String instanceName = value.getSecond();
		boolean isTagged = false;
		
		setText(instanceName);
		
		if(interactivityContext != null && interactivityContext.getManager() != null) {
			SortedSet<EID> events = interactivityContext.getManager().getAllTaggedEvents();
			
			//TODO we should be able to do this more efficiently
			for(EID id : events) {
				
				if(id.traceIdx == traceIdx) {
					isTagged = true;
					break;
				}
			}
		}
		
		if(isSelected && isTagged) {
			setBackground(COLOR_LIST_TAGGED_SELECTION_BG);
		}
		else if(isSelected) {
			setBackground(LogPreviewUI.COLOR_LIST_SELECTION_BG);
		}
		else if(isTagged) {
			setBackground(COLOR_LIST_TAGGED_BG);
		}
		else {
			setBackground(LogPreviewUI.COLOR_LIST_BG);
		}
		
		return this;
	}

}
