/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.log.ui.logdialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractiveVisualization;
import org.processmining.plugins.interactivevisualization.InteractivityContext;
import org.processmining.plugins.interactivevisualization.InteractivityManager;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class SlickerOpenLogSettings extends JPanel implements ScalableComponent, InteractiveVisualization {
	
	private static final long serialVersionUID = 4258730373866637365L;
	
	protected InspectorUI inspectorUI;
	protected LogDashboardUI dashboardUI;
	protected LogInfoUI loginfoUI;
	
	private final static String DASHBOARDICON = "dashboard48.png";
	private final static String INSPECTORICON = "inspector48.png";
	private final static String SUMMARYICON = "summary48-2.png";
	// icons
	protected Image dashboardIcon = new ImageIcon(SlickerOpenLogSettings.class.getResource(DASHBOARDICON),DASHBOARDICON).getImage();
	protected Image inspectorIcon = new ImageIcon(SlickerOpenLogSettings.class.getResource(INSPECTORICON),INSPECTORICON).getImage();
	protected Image summaryIcon = new ImageIcon(SlickerOpenLogSettings.class.getResource(SUMMARYICON),SUMMARYICON).getImage();
	
	
	
	// data attributes
	protected XLog log;
	protected XLogInfo summary;
	
	protected SlickerTabbedPane tabbedPane;
	
	// variables used in ScalableComponent interface.
	private final List<UpdateListener> observers = new LinkedList<UpdateListener>();
	
	// variables used in InteractiveVisualization interface.
	private InteractivityContext interactivityContext = new InteractivityContext();
	
	@Plugin(name = "@2 Log Summary", level = PluginLevel.PeerReviewed, parameterLabels = "Log", returnTypes = JComponent.class, returnLabels = "Log Visualization", userAccessible = true, mostSignificantResult = 1, help = "Provides an overview of the log")
	@Visualizer(name = "Log Visualizer")
	public JComponent showLogVis(PluginContext context, XLog log) {
		return showLogVis(context, log, XLogInfoFactory.createLogInfo(log), null);
	}
	
	@Plugin(name = "@1 Log Summary", parameterLabels = {"Log", "Image path"}, returnTypes = JComponent.class, returnLabels = "Log Visualization", userAccessible = true, mostSignificantResult = 1, help = "Provides an overview of the log")
	@Visualizer(name = "Log Visualizer")
	public JComponent showLogVis(PluginContext context, XLog log, String path) {
		return showLogVis(context, log, XLogInfoFactory.createLogInfo(log), path);
	}
	
	@Plugin(name = "@1 Log Summary", parameterLabels = {"Log", "Log summary"}, returnTypes = JComponent.class, returnLabels = "Log Visualization", userAccessible = true, mostSignificantResult = 1, help = "Provides an overview of the log")
	@Visualizer(name = "Log Visualizer using provided log info")
	public JComponent showLogVis(PluginContext context, XLog log, XLogInfo summary) {
		return showLogVis(context, log, summary, null);
	}
	
	@Plugin(name = "@0 Log Summary", parameterLabels = {"Log", "Log summary", "Image path"}, returnTypes = JComponent.class, returnLabels = "Log Visualization", userAccessible = true, mostSignificantResult = 1, help = "Provides an overview of the log")
	@Visualizer(name = "Log Visualizer using provided log info")
	public JComponent showLogVis(PluginContext context, XLog log, XLogInfo summary, String path) {
		this.log = log;
		this.summary = summary;
		initialize(path);
		completeGui(context);
		return this;
	}
	
	/**
	 * loads the log and initializes this component
	 */
	protected void initialize(String path) {
//		String promDir = System.getProperty("user.dir");
//		if (promDir.endsWith(".")) {
//			promDir = promDir.substring(0, promDir.length() - 1);
//		}
//		if (promDir.endsWith(System.getProperty("file.separator")) == false) {
//			promDir += System.getProperty("file.separator");
//		}
//		if (path == null) {
//			dashboardIcon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("dashboard48.png"))
//					.getImage();
//			inspectorIcon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("inspector48.png"))
//					.getImage();
//			summaryIcon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("summary48-2.png"))
//					.getImage();
//		} else {
//            String imagePath;
//            if (!path.endsWith(File.separator)) {
//            	imagePath = path + File.separator;
//            } else {
//            	imagePath = path;
//            }
//            dashboardIcon = new ImageIcon(imagePath + "dashboard48.png").getImage();
//            inspectorIcon = new ImageIcon(imagePath + "inspector48.png").getImage();
//            summaryIcon = new ImageIcon(imagePath + "summary48-2.png").getImage();
//		}
		setBackground(new Color(40, 40, 40));

		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());

	}
	
	protected void completeGui(PluginContext context) {
		// compile inspector and dashboard panels
		inspectorUI = new InspectorUI(this, interactivityContext);
		dashboardUI = new LogDashboardUI(this, interactivityContext);
		loginfoUI = new LogInfoUI(this, context);
		
		// compile root panel
		IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,
				20, 160));
		iconTabs.addTab("Dashboard", dashboardIcon, dashboardUI);
		iconTabs.addTab("Inspector", inspectorIcon, inspectorUI);
		iconTabs.addTab("Summary", summaryIcon, loginfoUI);
		add(iconTabs, BorderLayout.CENTER);
		revalidate();
		repaint();
	}
	
	public XLog getLog() {
		return log;
	}
	
	public XLogInfo getLogInfo() {
		return summary;
	}
	
	/*
	 * Implementation of Interactive Visualization interface (non-Javadoc)
	 * @see org.processmining.plugins.interactivevisualization.InteractiveVisualization
	 */
	
	@Override
	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		
		if(logChanged) {
			
			if(interactivityContext != null && interactivityContext.getManager() != null) {
				summary = interactivityContext.getManager().getLogInfo();
			}
			else {
				//This should never happen, but just in case ...
				summary = XLogInfoFactory.createLogInfo(log);
			}
		}
		
		dashboardUI.updateVisualization(logChanged, tagsChanged);
		inspectorUI.updateVisualization(logChanged, tagsChanged);
		loginfoUI.updateVisualization(logChanged, tagsChanged);
		
		repaint();
	}
	
	@Override
	public void setManager(InteractivityManager manager) {
		interactivityContext.setManager(manager);
	}
	
	@Override
	public Map<String, SortedSet<EID>> getSelectedElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating selected elements.");
	}
	
	@Override
	public Map<String, SortedSet<EID>> getVisibleElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating visible elements.");
	}
	
	@Override
	public InteractivityManager getManager() {
		return interactivityContext.getManager();
	}
	
	/*
	 * Implementation of Scalable Component interface (non-Javadoc)
	 * @see org.processmining.framework.util.ui.scalableview.ScalableComponent
	 */
	
	@Override
	public JComponent getComponent() {
		return this;
	}
	
	@Override
	public double getScale() {
		return 1;
	}
	
	@Override
	public void setScale(double newScale) {
		//TODO debug code
		System.err.println("Supposed to set the scale to "+newScale);
		
		//TODO set scale
	}
	
	@Override
	public void addUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.add(listener);
	}
	
	@Override
	public void removeUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.remove(listener);
	}
}
